//Example program demonstrating the use of prepared selections and joins
//After the first join, the subsequent join operations becomes very cheap
#include "ffdb.h"
#include "ffdb_io.h"
#include "ffdb_op.h"
#include "ffdb_util.h"
#include <stdio.h>

using namespace ffdb;

int main(void) {
	RowSet rowset_user;
	if(!ReadFromFile("example_db_user.ffdb",&rowset_user)) {
		fprintf(stderr,"Could not read from user database\n");
		return -1;
	}
	
	RowSet rowset_quota;
	if(!ReadFromFile("example_db_quota.ffdb",&rowset_quota)) {
		fprintf(stderr,"Could not read from quota database\n");
		return -1;
	}
	
	//set up locked selection
	LockedSelection locked_sel;
	locked_sel.lock(rowset_user);
	int cond_idx = locked_sel.add_condition("username",cond_eq);
	
	//set up locked join
	LockedJoin locked_join;
	locked_join.lock2(rowset_quota);
	locked_join.add_relation("id","userid");
	
	//set up prepared projection
	PreparedProjection prep_projection;
	prep_projection.add_column("disk");
	prep_projection.add_column("quota");
	
	//now for rapid executaion of the releational operations
	RowSet result1;
	RowSet result2;
	RowSet result3;
	
	locked_sel.set_value(cond_idx,"johnny");
	locked_sel.execute(&result1);
	locked_join.lock1(result1);
	locked_join.execute(&result2);
	prep_projection.execute(result2,&result3);
	printf("Johnny's quotas:\n");
	dump(result3,stdout);
	
	locked_sel.set_value(cond_idx,"jim");
	locked_sel.execute(&result1);
	locked_join.lock1(result1);
	locked_join.execute(&result2);
	prep_projection.execute(result2,&result3);
	printf("Jim's quotas:\n");
	dump(result3,stdout);
	
	return 0;
}
