#include "ffdb_op.h"

namespace ffdb {

bool BinaryOpHelper::prepare_cache(const RowSetDescriptor &desc1, const RowSetDescriptor &desc2) {
	cached_column1_idx.clear();
	cached_column2_idx.clear();
	for(column_pair_t::size_type i=0; i<column_pair.size(); i++) {
		column_pair_t::size_type j1;
		for(j1=0; j1<desc1.size(); j1++) {
			if(desc1[j1].name==column_pair[i].first)
				break;
		}
		if(j1>=desc1.size()) return false;
		column_pair_t::size_type j2;
		for(j2=0; j2<desc2.size(); j2++) {
			if(desc2[j2].name==column_pair[i].second)
				break;
		}
		if(j2>=desc2.size()) return false;
		if(desc1[j1].type!=desc2[j2].type)
			return false;
		cached_column1_idx.push_back(j1);
		cached_column2_idx.push_back(j2);
	}
	return true;
}


void BinaryOpHelper::add_relation(const char *column_name_1, const char *column_name_2)
{
	column_pair.push_back(std::pair<std::string,std::string>(std::string(column_name_1),std::string(column_name_2)));
}


void BinaryOpHelper::process_rowset(Chunk &chunk,
                                    const DigestHelper::ColumnIdx &column_idx,
                                    bool sort, bool with_nulls)
{
	if(with_nulls && !chunk.rowset_null_rows_valid) {
		chunk.rowset_digest_valid = false;
	}
	if(!chunk.rowset_digest_valid) {
		if(with_nulls) {
			DigestHelper::generate_rowset_digest(*chunk.rowset,column_idx,&chunk.rowset_digest,&chunk.rowset_null_rows);
			chunk.rowset_null_rows_valid = true;
		} else {
			DigestHelper::generate_rowset_digest(*chunk.rowset,column_idx,&chunk.rowset_digest);
			chunk.rowset_null_rows_valid = true;
		}
		chunk.rowset_digest_valid = true;
	}
	if(sort && !chunk.rowset_digest_sorted) {
		DigestHelper::sort_rowset_digest(&chunk.rowset_digest);
		chunk.rowset_digest_sorted = true;
	}
}


void BinaryOpHelper::lock(Chunk &chunk, const RowSet &rowset)
{
	unlock(chunk);
	chunk.rowset = &rowset;
}


void BinaryOpHelper::unlock(Chunk &chunk)
{
	chunk.rowset = 0;
	chunk.rowset_digest_valid = false;
	chunk.rowset_digest_sorted = false;
	chunk.rowset_digest.clear();
	chunk.rowset_null_rows_valid = false;
	chunk.rowset_null_rows.clear();
}


BinaryOpHelper::~BinaryOpHelper() {
}

} //namespace
