#include "ffdb_op_rowsetdigest.h"
#include <string.h>
#include <stdlib.h>
#include <new>

namespace ffdb {

//RowSetDigest is used for indexing, uniqueness, etc. It gets a subset of hte columns, and stores them into 'buffer'.

RowSetDigest::RowSetDigest()
  : buffer(0),
    buffer_size(0),
    buffer_bytes(0)
{ }


RowSetDigest::~RowSetDigest() {
	clear();
}

void RowSetDigest::reserve_raw(size_t rawsize) {
	if(rawsize>buffer_size) {
		size_t newsize = buffer_size + buffer_size/2;
		if(newsize<rawsize) newsize=rawsize;
		
		char *newbuf = (char*)realloc(buffer,newsize);
		if(!newbuf) throw std::bad_alloc();
		
		//lint --e{449,613}  We actually know what we are doing here... Don't try this at home.
		ptrdiff_t address_delta = newbuf - buffer;
		for(std::vector<RowDigest>::size_type i=0; i<digest.size(); i++)
			digest[i].ptr += address_delta;
		buffer = newbuf;
		buffer_size = newsize;
	}
}


void RowSetDigest::digest_alloc(RowSet::const_pointer row_ptr) {
	digest.push_back(RowDigest());
	RowDigest *d=&(*(digest.end())) - 1;
	d->ptr = buffer + buffer_bytes; //lint !e613
	d->size = 0;
	d->row_ptr = row_ptr;
}


void RowSetDigest::digest_append_last(const void *p, size_t bytes) {
	RowDigest *d=&(*(digest.end())) - 1;
	d->size += bytes;
	reserve_raw(buffer_bytes + bytes);
	memcpy(buffer+buffer_bytes, p, bytes);
	buffer_bytes += bytes;
}


void RowSetDigest::clear() throw() {
	free(buffer);
	buffer=0;
	buffer_bytes=0;
	buffer_size=0;
	digest.clear(); //lint !e1550
}

} //namespace
