#include "ffdb.h"
#include "ffdb_op.h"
#include "ffdb_op_cmpdigest.h"
#include <stdarg.h>

namespace ffdb {

void PreparedUnique::add_column(const char *column_name_) {
	column_name.push_back(std::string(column_name_));
}


bool PreparedUnique::execute(const RowSet &rowset, RowSet *result) {
	const RowSetDescriptor &desc = rowset.descriptor();
	DigestHelper::ColumnIdx column_idx;
	if(!column_name.empty()) {
		//find column indexes
		for(columnidx_t i=0; i<column_name.size(); i++) {
			columnidx_t j;
			for(j=0; j<desc.size(); j++)
				if(desc[j].name == column_name[i])
					break;
			if(j>=desc.size())
				return false;
			column_idx.push_back(j);
		}
	} else {
		for(columnidx_t i=0; i<desc.size(); i++)
			column_idx.push_back(i);
	}
	
	//
	RowSetDigest digest;
	std::vector<RowSet::const_pointer> rowset_null_rows;
	DigestHelper::generate_rowset_digest(rowset,column_idx,&digest,&rowset_null_rows);
	
	DigestHelper::sort_rowset_digest(&digest);

	result->setDescriptor(desc);
	result->clear();
	
	unsigned rows_processed=0;
	Compare_RowDigest cmp;
	for(RowSetDigest::const_iterator iter=digest.begin();
	    iter!=digest.end();
	    ++iter)
	{
		if(iter==digest.begin() || cmp.ne(iter[0],iter[-1])) //lint !e747
			result->push_back(*iter->row_ptr);
		else
			; //nothing
		rows_processed++;
		if(rows_processed==1000) {
			double ratio = result->size()/1000.0;
			size_t est_result_rows = (size_t)(digest.size() * ratio) + rowset_null_rows.size();
			if(est_result_rows > rowset.size()) est_result_rows = rowset.size();
			result->reserve(est_result_rows);
		}
	}
	result->reserve(result->size() + rowset_null_rows.size());
	for(std::vector<RowSet::const_pointer>::const_iterator i=rowset_null_rows.begin();
	    i!=rowset_null_rows.end();
	    ++i)
		result->push_back(**i);
	
	return true;
}




bool Unique(const RowSet &rowset, RowSet *result, const char *column_name_0, ...) {
	PreparedUnique prep_unique;
	va_list ap;
	va_start(ap,column_name_0);
	
	const char *cn=column_name_0;
	while(cn) {
		prep_unique.add_column(cn);
		cn = va_arg(ap,const char*);
	}
	va_end(ap);
	
	return prep_unique.execute(rowset,result);
}

} //namespace
