#define __STDC_FORMAT_MACROS
#include "ffdb_util.h"
#include <inttypes.h>

namespace ffdb {

void dump(const RowSet &rowset, FILE *fp) {
	const RowSetDescriptor &desc = rowset.descriptor();
	for(columnidx_t i=0; i<desc.size(); i++) {
		if(i!=0) fprintf(fp,"\t");
		fprintf(fp,"%s",desc[i].name.c_str());
	}
	fprintf(fp,"\n");
	
	for(RowSet::const_iterator i = rowset.begin();
	    i!=rowset.end();
	    ++i
	   )
	{
		Row::const_iterator rowcolumn_iter = i->begin();
		for(columnidx_t c=0; c<i->size(); c++) {
			if(c!=0) fprintf(fp,"\t");
			size_t label_width = desc[c].name.size();
			const void *ptr = rowcolumn_iter->ptr;
			if(ptr) {
				switch(desc[c].type) {
					case ColumnType_integer: {
						int64_t value = *(int64_t*)ptr;
						fprintf(fp,"%*"PRId64, (int)label_width,value);
						break;
					}
					case ColumnType_real: {
						double value = *(double*)ptr;
						fprintf(fp,"%f", value);
						break;
					}
					case ColumnType_string: {
						size_t sl=rowcolumn_iter->size;
						fprintf(fp,"%-*.*s",(int)label_width,(int)sl,(char*)ptr);
						break;
					}
					case ColumnType_raw: {
						size_t sl=rowcolumn_iter->size;
						const uint8_t *p=(const uint8_t*)ptr;
						for(size_t j=0; j<sl; j++) {
							fprintf(fp,"%02"PRIx8, p[j]);
						}
						break;
					}
					case ColumnType_boolean: {
						bool value = *(bool*)ptr;
						const char *v = value?"true":"false";
						fprintf(fp,"%-*s",(int)label_width,v);
						break;
					}
				}
			} else {
				fprintf(fp,"%*.*s",(int)label_width,(int)label_width,"NULL");
			}
			++rowcolumn_iter;
		}
		fprintf(fp,"\n");
	}
}

} //namespace
