#include "ffdb.h"
#include <assert.h>
//#include <string.h>

using namespace ffdb;

int main(void) {
	//---------
	//Row
	{
		Row r1;
		
		assert(r1.raw_size()==0);
		assert(r1.size()==0);
		assert(r1.empty());
		assert(r1.begin()==r1.end());
		assert(r1.rbegin()==r1.rend());
		assert(r1==r1);
		assert(!(r1!=r1));
		
		Row r2;
		assert(r1==r2);
		assert(!(r1!=r2));
	}
	{
		Row r1;
		r1.alloc(4,1,"foo");
		assert(r1.raw_size()==4);
		assert(r1.size()==1);
		assert(!r1.empty());
		assert(r1.begin()!=r1.end());
		assert(r1.rbegin()!=r1.rend());
		
		Row r2;
		assert(r1!=r2);
		assert(!(r1==r2));
		
		r2=r1;
		assert(r2==r1);
		assert(!(r2!=r1));
		
		r1.clear();
		assert(r1.empty());
		assert(r1!=r2);
		
		r1.alloc(4,1,"foo");
		r1.alloc(4,1,"boo");
		assert(r1.size()==2);
		
		assert(r2.size()!=2);
		r1.swap(r2);
		assert(r2.size()==2);
		swap(r1,r2);
		assert(r1.size()==2);
		assert(r2.size()!=2);
	}
	
	{
		Row r1;
		r1.alloc(1,1,"");
		assert(r1.raw_size()==1);
		r1.alloc(2,2,"x");
		assert(r1.raw_size()==4);
		r1.alloc(1,1,"");
		assert(r1.raw_size()==5);
		r1.alloc(4,4,"xxx");
		assert(r1.raw_size()==12);
		r1.alloc(8,8,"xxxxxxx");
		assert(r1.raw_size()==24);
		r1.alloc_null();
		assert(r1.raw_size()==24);
	}
	
	
	//------------
	//ColumnDescriptor
	{
		ColumnDescriptor cd1(ColumnType_integer,"foo");
		assert(cd1.type==ColumnType_integer);
		assert(cd1.name=="foo");
		assert(cd1==cd1);
		assert(!(cd1!=cd1));
	}
	{
		ColumnDescriptor cd1(ColumnType_integer,"foo");
		ColumnDescriptor cd2(ColumnType_real,"boo");
		ColumnDescriptor cd3(ColumnType_real,"foo");
		ColumnDescriptor cd4(ColumnType_integer,"boo");
		ColumnDescriptor cd5(ColumnType_integer,"foo");
		assert(cd1!=cd2);
		assert(!(cd1==cd2));
		assert(cd1!=cd3);
		assert(!(cd1==cd3));
		assert(cd1!=cd4);
		assert(!(cd1==cd4));
		assert(cd1==cd5);
		assert(!(cd1!=cd5));
	}
	
	
	//-------
	//RowSetDescriptor
	{
		RowSetDescriptor rsd1;
		assert(rsd1.empty());
		assert(rsd1.size()==0);
		assert(rsd1==rsd1);
		assert(!(rsd1!=rsd1));
		
		rsd1.push_back(ColumnDescriptor(ColumnType_integer,"id"));
		rsd1.push_back(ColumnDescriptor(ColumnType_string,"name"));
		assert(rsd1.size()==2);
		
		RowSetDescriptor rsd2;
		rsd2.push_back(ColumnDescriptor(ColumnType_integer,"short_id"));
		rsd2.push_back(ColumnDescriptor(ColumnType_string,"short_name"));
		
		assert(is_type_compatible(rsd1,rsd2));
		
		RowSetDescriptor rsd3;
		rsd3.push_back(ColumnDescriptor(ColumnType_real,"id"));
		rsd3.push_back(ColumnDescriptor(ColumnType_string,"name"));
		
		assert(!is_type_compatible(rsd1,rsd3));
	}
	
	
	//--------
	//RowSet (incomplete unittest)
	{
		RowSet rs1;
		assert(rs1.empty());
		assert(rs1.size()==0);
		assert(rs1.begin()==rs1.end());
		
		RowSetDescriptor rsd1;
		rsd1.push_back(ColumnDescriptor(ColumnType_integer,"id"));
		rsd1.push_back(ColumnDescriptor(ColumnType_string,"name"));
		
		rs1.setDescriptor(rsd1);
		assert(rs1.descriptor()==rsd1);
		
		Row r1;
		int64_t id=42;
		const char name[]="foo";
		r1.alloc(8,8,&id);
		r1.alloc(sizeof(name),1,name);
		rs1.push_back(r1);
		
		assert(!rs1.empty());
		assert(rs1.size()==1);
		assert(rs1.begin()!=rs1.end());
	}
}
