#include "ffdb.h"
#include "ffdb_io.h"
#include "ffdb_row_access.h"
#include <stdio.h>

using namespace ffdb;

#define FAIL(what) { fprintf(stderr,"Failed " #what "\n"); return -1; }

int main(void) {
	FILE *fp;
	
	//empty files with various kinds of headers
	fp=fopen("unittest_io2.csv","w");
	fclose(fp);
	{
		RowSet rowset;
		if(!ReadFromFile("unittest_io2.csv",&rowset,header_format_none,"NULL",','))
			FAIL("empty-file+no-header test");
	}
	{
		RowSetDescriptor desc;
		desc.push_back(ColumnDescriptor(ColumnType_integer,"id"));
		RowSet rowset;
		rowset.setDescriptor(desc);
		
		if(!ReadFromFile("unittest_io2.csv",&rowset,header_format_none,"NULL",','))
			FAIL("empty-file+no-header test");
	}
	{
		RowSet rowset;
		if(ReadFromFile("unittest_io2.csv",&rowset,header_format_names_only,"NULL",','))
			FAIL("empty-file+names-only-header test");
	}
	
	//header-only
	fp=fopen("unittest_io2.csv","w");
	fprintf(fp,"id,name\n");
	fclose(fp);
	{
		RowSet rowset;
		if(ReadFromFile("unittest_io2.csv",&rowset,header_format_names_only,"NULL",','))
			FAIL("header-only+no-descriptor");
	}
	{
		RowSetDescriptor desc;
		desc.push_back(ColumnDescriptor(ColumnType_integer,"foo"));
		desc.push_back(ColumnDescriptor(ColumnType_string,"boo"));
		RowSet rowset;
		rowset.setDescriptor(desc);
		if(!ReadFromFile("unittest_io2.csv",&rowset,header_format_names_only,"NULL",','))
			FAIL("header-only+no-descriptor (1)");
		if(rowset.descriptor().size()!=2)
			FAIL("header-only+no-descriptor (2)");
		if(rowset.descriptor()[0].name!="id")
			FAIL("header-only+no-descriptor (3)");
		if(rowset.descriptor()[1].name!="name")
			FAIL("header-only+no-descriptor (4)");
	}
// 	{
// 		RowSetDescriptor desc;
// 		desc.push_back(ColumnDescriptor(ColumnType_integer,"foo"));
// 		desc.push_back(ColumnDescriptor(ColumnType_string,"boo"));
// 		RowSet rowset;
// 		rowset.setDescriptor(desc);
// 		if(!ReadFromFile("unittest_io2.csv",&rowset,header_format_none,"NULL",','))
// 			FAIL("header-only+no-descriptor (none) (1)");
// 		if(rowset.descriptor().size()!=2)
// 			FAIL("header-only+no-descriptor (none) (2)");
// 		if(rowset.descriptor()[0].name!="foo")
// 			FAIL("header-only+no-descriptor (none) (3)");
// 		if(rowset.descriptor()[1].name!="boo")
// 			FAIL("header-only+no-descriptor (none) (4)");
// 	}
	{
		RowSetDescriptor desc;
		desc.push_back(ColumnDescriptor(ColumnType_string,"foo"));
		RowSet rowset;
		rowset.setDescriptor(desc);
		if(!ReadFromFile("unittest_io2.csv",&rowset,header_format_names_only,"NULL",'@'))
			FAIL("header-only+no-descriptor (@) (1)");
		if(rowset.descriptor().size()!=1)
			FAIL("header-only+no-descriptor (@) (2)");
		if(rowset.descriptor()[0].name!="id,name")
			FAIL("header-only+no-descriptor (@) (3)");
	}
	
	//simple, valid file
	fp=fopen("unittest_io2.csv","w");
	fprintf(fp,"id,name\n");
	fprintf(fp,"1,a_name\n");
	fprintf(fp,"2,NULL\n");
	fprintf(fp,"3,a name\n");
	fprintf(fp,"4,\n");
	fprintf(fp,"5\n");
	fprintf(fp,"6,\"a,name\"\n");
	fclose(fp);
	{
		RowSetDescriptor desc;
		desc.push_back(ColumnDescriptor(ColumnType_integer,"foo"));
		desc.push_back(ColumnDescriptor(ColumnType_string,"boo"));
		RowSet rowset;
		rowset.setDescriptor(desc);
		if(!ReadFromFile("unittest_io2.csv",&rowset,header_format_names_only,"NULL",','))
			FAIL("simple test (1)");
		if(rowset.descriptor().size()!=2)
			FAIL("simple test (2)");
		if(rowset.descriptor()[0].name!="id")
			FAIL("simple test (3)");
		if(rowset.descriptor()[1].name!="name")
			FAIL("simple test (4)");
		if(rowset.size()!=6)
			FAIL("simple test (6)");
		RowSet::iterator i=rowset.begin();
		if(getInteger(rowset.descriptor(),i,"id")!=1)
			FAIL("simple test 7.1");
		if(strcmp(getString(rowset.descriptor(),i,"name"),"a_name")!=0)
			FAIL("simple test 7.2");
		++i;
		if(getInteger(rowset.descriptor(),i,"id")!=2)
			FAIL("simple test 7.3");
		if(!isNull(rowset.descriptor(),i,"name"))
			FAIL("simple test 7.4");
		++i;
		if(getInteger(rowset.descriptor(),i,"id")!=3)
			FAIL("simple test 7.5");
		if(strcmp(getString(rowset.descriptor(),i,"name"),"a name")!=0)
			FAIL("simple test 7.6");
		++i;
		if(getInteger(rowset.descriptor(),i,"id")!=4)
			FAIL("simple test 7.7");
		if(strcmp(getString(rowset.descriptor(),i,"name"),"")!=0)
			FAIL("simple test 7.8");
		++i;
		if(getInteger(rowset.descriptor(),i,"id")!=5)
			FAIL("simple test 7.9");
		if(!isNull(rowset.descriptor(),i,"name"))
			FAIL("simple test 7.10");
		++i;
		if(getInteger(rowset.descriptor(),i,"id")!=6)
			FAIL("simple test 7.11");
		if(strcmp(getString(rowset.descriptor(),i,"name"),"a,name")!=0)
			FAIL("simple test 7.12");
	}
}
