#include "ffdb_op.h"
#include <stdlib.h>
#include "ffdb_op_cmpdigest.h"

namespace ffdb {


void DigestHelper::generate_rowset_digest(const RowSet &rowset, 
                                          const ColumnIdx &column_idx, 
                                          RowSetDigest *digest,
                                          std::vector<RowSet::const_pointer> *rowset_null_rows)
{
	digest->reserve(rowset.size());
	unsigned row_index=0;
	for(RowSet::const_iterator iter=rowset.begin();
	    iter!=rowset.end();
	    ++iter)
	{
		digest->digest_alloc(&*iter);
		for(ColumnIdx::size_type j=0; j<column_idx.size(); j++) {
			ColumnIdx::value_type k=column_idx[j];
			const Row::VL &vl = (*iter)[k];
			if(!vl.ptr) {
				rowset_null_rows->push_back(&*iter);
				digest->pop_back();
				break; //cannot join on NULLs
			}
			if(rowset.descriptor()[k].type==ColumnType_raw)
				digest->digest_append_last(&vl.size,sizeof(vl.size));
			digest->digest_append_last(vl.ptr,vl.size);
		}
		row_index++;
		if(row_index==1000) {
			//Enough for an estimate
			double est_total_rows = digest->raw_size() * 1000.0/rowset.size();
			digest->reserve_raw((size_t)est_total_rows);
		}
	}
}


void DigestHelper::generate_rowset_digest(const RowSet &rowset, 
                                          const ColumnIdx &column_idx, 
                                          RowSetDigest *digest)
{
	digest->reserve(rowset.size());
	unsigned row_index=0;
	for(RowSet::const_iterator iter=rowset.begin();
	    iter!=rowset.end();
	    ++iter)
	{
		digest->digest_alloc(&*iter);
		for(ColumnIdx::size_type j=0; j<column_idx.size(); j++) {
			ColumnIdx::value_type k=column_idx[j];
			const Row::VL &vl = (*iter)[k];
			if(!vl.ptr) {
				digest->pop_back();
				break; //cannot join on NULLs
			}
			digest->digest_append_last(vl.ptr,vl.size);
		}
		row_index++;
		if(row_index==1000) {
			//Enough for an estimate
			double est_total_rows = digest->raw_size() * 1000.0/rowset.size();
			digest->reserve_raw((size_t)est_total_rows);
		}
	}
}



//qsort() is faster than std::sort
//Probably because qsort() can exploit more return values from the compare
//function and because qsort() is a very old c library thing and likely to
//be heavily optimized. std::sort usually isn't
//
//An interesting excercise would be to replace qsort() with a radix sort.
//Or scan the digest array for minimum length and doing a 1/2/4-byte bucket
//sort followed by a regular selection sort.
//Or probe the first byt of the digest (they cannot be zero-length) if
//memcmp() is too slow.
//Or probe the first word on big-endian machines. Should be faster than
//memcmp() (requires that we word-align the digests)

extern "C" {
static int cmp_RowDigest(const void *pv1, const void *pv2) {
	const RowSetDigest::RowDigest *rd1 = (const RowSetDigest::RowDigest*)pv1;
	const RowSetDigest::RowDigest *rd2 = (const RowSetDigest::RowDigest*)pv2;
	size_t sz1 = rd1->size;
	size_t sz2 = rd2->size;
	size_t ms = sz1;
	if(sz2<ms) ms=sz2;
	int rc = memcmp(rd1->ptr,rd2->ptr,ms);
	if(rc!=0) return rc;
	if(sz1<sz2) return -1;
	if(sz1>sz2) return 1;
	return 0;
}
}


void DigestHelper::sort_rowset_digest(RowSetDigest *digest) {
	qsort(&(*(digest->begin())), digest->size(), sizeof(RowSetDigest::RowDigest), cmp_RowDigest);
}

} //namespace
